
import os
import sys

import distutils.core

if len(sys.argv) < 1:
    raise SystemExit("Need at least the package to upload")

#
# run_setup starts here. It was copied because we needn't pass locals dict to
# execfile
distutils.core._setup_stop_after = 'init'
script_name = 'setup.py'

save_argv = sys.argv
g = {
    '__file__': script_name, # for those who call os.path.dirname(__file__)
    '__name__': '__main__'   # for those who have if __name__ == '__main__':
}

l = {}
try:
    try:
        sys.argv = [script_name]
        execfile(script_name,g)
    finally:
        sys.argv = save_argv
        _setup_stop_after = None
except SystemExit:
    pass
except:
    raise

if distutils.core._setup_distribution is None:
    raise RuntimeError, \
          ("'distutils.core.setup()' was never called -- "
           "perhaps '%s' is not a Distutils setup script?") % \
          script_name

dist = distutils.core._setup_distribution
# run_setup ends here
#

dist.commands = ['upload']
dist.dist_files = [('command', 'version', os.path.abspath(sys.argv[1]))]
dist.command_options = {
    'upload': {'repository': ('command line', 'spynepi')},
}

dist.run_commands()



# encoding: utf8
#
# (C) Copyright Arskom Ltd. <info@arskom.com.tr>
#               Uğurcan Ergün <ugurcanergn@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#


import logging
logger = logging.getLogger(__name__)

import os
import sys
import shutil
import tempfile
import subprocess

from glob import glob

from pkg_resources import Requirement

from setuptools.archive_util import unpack_archive
from setuptools.package_index import PackageIndex

from spyne.error import ValidationError
from spyne.error import ArgumentError
from spynepi.const import REPO_NAME


def _generate_pypirc(own_url):
    import ConfigParser

    rc = os.path.join(os.environ['HOME'], '.pypirc')
    config = ConfigParser.ConfigParser()

    if os.path.exists(rc):
        config.read(rc)

    try:
        config.add_section(REPO_NAME)

        config.set(REPO_NAME, 'repository', own_url)
        config.set(REPO_NAME, 'username', 'x')
        config.set(REPO_NAME, 'password', 'y')

    except ConfigParser.DuplicateSectionError:
        pass

    try:
        config.add_section('distutils')
    except ConfigParser.DuplicateSectionError:
        pass

    try:
        index_servers = config.get('distutils', 'index-servers')
        index_servers = index_servers.split('\n')
        if 'spynepi' not in index_servers:
            index_servers.append(REPO_NAME)

    except ConfigParser.NoOptionError:
        index_servers = [REPO_NAME]

    config.set('distutils', 'index-servers', '\n'.join(index_servers))

    config.write(open(rc,'w'))


def cache_package(spec, own_url):
    try:
        spec = Requirement.parse(spec)

    except ValueError:
        raise ArgumentError("Not a URL, existing file, or requirement spec: %r"
                                                                      % (spec,))

    try:
        # download and unpack source package
        path = tempfile.mkdtemp('.spynepi')
        logger.info("Downloading %r" % spec)
        dist = PackageIndex().fetch_distribution(spec, path, force_scan=True, source=True)
        archive_path = dist.location
        logger.info("Unpacking %r" % archive_path)
        unpack_archive(dist.location, path)

        # generate pypirc if possible
        if os.environ.has_key('HOME'):
            _generate_pypirc(own_url)
        else: # FIXME: ??? No idea. Hopefully setuptools knows better.
            pass # raise NotImplementedError("$HOME not defined, .pypirc not found.")

        # find setup.py in package. plagiarized from setuptools.
        setups = glob(os.path.join(path, '*', 'setup.py'))
        if not setups:
            raise ValidationError(
                "Couldn't find a setup script in %r editable distribution: %r" %
                                    (spec, os.path.join(path,'*', 'setup.py'))
            )

        if len(setups)>1:
            raise ValidationError(
                "Multiple setup scripts in found in %r editable distribution: %r" %
                                    (spec, setups)
            )

        # self-register the package.
        lib_dir = os.path.dirname(setups[0])
        command = ["python", "setup.py", "register", "-r", REPO_NAME]
        logger.info('calling %r', command)
        subprocess.call(command, cwd=lib_dir, stdout=sys.stdout)

        # self-upload the package
        command = ["python", "-m", "spynepi.util.pypi.upload", archive_path]
        logger.info('calling %r', command)
        subprocess.call(command, cwd=lib_dir, stdin=sys.stdin, stdout=sys.stdout)

    finally:
        shutil.rmtree(path)




#
# Shamelessly ripped off from:
#
# https://bitbucket.org/loewis/pypi/src/b3c267d4ca696ca94427cff1e832238d0bf6be5f/description_utils.py?at=default
#


import sys
import StringIO
import cgi
import urlparse

import docutils.utils # workaround that makes the next import work.

from docutils import io
from docutils import readers
from docutils.core import Publisher
from docutils.core import publish_doctree
from docutils.transforms import TransformError


def trim_docstring(text):
    """
    Trim indentation and blank lines from docstring text & return it.

    See PEP 257.
    """
    if not text:
        return text
    # Convert tabs to spaces (following the normal Python rules)
    # and split into a list of lines:
    lines = text.expandtabs().splitlines()
    # Determine minimum indentation (first line doesn't count):
    indent = sys.maxint
    for line in lines[1:]:
        stripped = line.lstrip()
        if stripped:
            indent = min(indent, len(line) - len(stripped))
    # Remove indentation (first line is special):
    trimmed = [lines[0].strip()]
    if indent < sys.maxint:
        for line in lines[1:]:
            trimmed.append(line[indent:].rstrip())
    # Strip off trailing and leading blank lines:
    while trimmed and not trimmed[-1]:
        trimmed.pop()
    while trimmed and not trimmed[0]:
        trimmed.pop(0)
    # Return a single string:
    return '\n'.join(trimmed)


ALLOWED_SCHEMES = '''file ftp gopher hdl http https imap mailto mms news nntp
prospero rsync rtsp rtspu sftp shttp sip sips snews svn svn+ssh telnet
wais'''.split()


def process_description(source, output_encoding='unicode'):
    """Given an source string, returns an HTML fragment as a string.

    The return value is the contents of the <body> tag.

    Parameters:

    - `source`: A multi-line text string; required.
    - `output_encoding`: The desired encoding of the output.  If a Unicode
      string is desired, use the default value of "unicode" .
    """
    # Dedent all lines of `source`.
    source = trim_docstring(source)

    settings_overrides={
        'raw_enabled': 0,  # no raw HTML code
        'file_insertion_enabled': 0,  # no file/URL access
        'halt_level': 2,  # at warnings or errors, raise an exception
        'report_level': 5,  # never report problems with the reST code
        }

    # capture publishing errors, they go to stderr
    old_stderr = sys.stderr
    sys.stderr = s = StringIO.StringIO()
    parts = None

    try:
        # Convert reStructuredText to HTML using Docutils.
        document = publish_doctree(source=source,
            settings_overrides=settings_overrides)

        for node in document.traverse():
            if node.tagname == '#text':
                continue
            if node.hasattr('refuri'):
                uri = node['refuri']
            elif node.hasattr('uri'):
                uri = node['uri']
            else:
                continue
            o = urlparse.urlparse(uri)
            if o.scheme not in ALLOWED_SCHEMES:
                raise TransformError('link scheme not allowed')

        # now turn the transformed document into HTML
        reader = readers.doctree.Reader(parser_name='null')
        pub = Publisher(reader, source=io.DocTreeInput(document),
            destination_class=io.StringOutput)
        pub.set_writer('html')
        pub.process_programmatic_settings(None, settings_overrides, None)
        pub.set_destination(None, None)
        pub.publish()
        parts = pub.writer.parts

    except:
        pass

    sys.stderr = old_stderr

    # original text if publishing errors occur
    if parts is None or len(s.getvalue()) > 0:
        output = "".join('<PRE>\n' + cgi.escape(source) + '</PRE>')
    else:
        output = parts['body']

    if output_encoding != 'unicode':
        output = output.encode(output_encoding)

    return output



# encoding: utf8
#
# (C) Copyright Arskom Ltd. <info@arskom.com.tr>
#               Uğurcan Ergün <ugurcanergn@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#

from spynepi.util.description import process_description
import logging
logger = logging.getLogger(__name__)

import os
import datetime

from sqlalchemy import sql

from spyne.decorator import rpc
from spyne.error import ArgumentError
from spyne.model.primitive import String
from spyne.model.primitive import Unicode
from spyne.service import ServiceBase

from spyne.model.binary import File
from spyne.protocol.http import HttpPattern

from spynepi.db import Package
from spynepi.db import Release
from spynepi.db import Distribution
from spynepi.db import Person


class RootService(ServiceBase):
    FILES_ROOT = None # To be set by main()

    @rpc(Unicode,#name
        Unicode, #license
        Unicode, #author
        Unicode, #home_page
        File,    #content
        Unicode, #comment
        Unicode, #download_url
        Unicode, #platform
        Unicode, #description
        Unicode, #metadata_version
        Unicode, #author_email
        Unicode, #md5_digest
        Unicode, #filetype
        Unicode, #pyversion
        Unicode, #summary
        Unicode, #version
        Unicode, #protocol_version
        String(encoding='ascii'), #action
        _patterns=[HttpPattern("/",verb="POST")],
        _in_variable_names={'action': ':action'})

    def register(ctx, name, license, author, home_page, content, comment,
            download_url, platform, description, metadata_version, author_email,
            md5_digest, filetype, pyversion, summary, version, protcol_version, # <= this is not a typo
            action):

        pth = os.path.join(name, version)

        def generate_package():
            return Package(
                    package_name=name,
                    package_cdate=datetime.date.today(),
                    package_description=process_description(description, 'utf8'),
                    rdf_about=os.path.join("/", name),
                    package_license=license,
                    package_home_page=home_page
                )

        def generate_person():
            return Person(person_name=author,
                    person_email=author_email,
                )

        def generate_release():
            return Release(
                    rdf_about=os.path.join("/", name, version),
                    release_version=version,
                    release_cdate=datetime.date.today(),
                    release_summary=summary ,
                    meta_version=metadata_version,
                    release_platform=platform,
                )

        def write_package_content(file_path):
            if not os.path.exists(path):
                os.makedirs(os.path.dirname(file_path))

            f = open(file_path,"w")

            for d in content.data:
                f.write(d)
            f.close()

        package = ctx.udc.session.query(Package).filter_by(package_name=name).first()
        release = None
        if package is not None:
            release = ctx.udc.session.query(Release).join(Package).filter(
                sql.and_(
                    Package.package_name == name,
                    Release.release_version == version,
                )).first()

        if action == "submit":
            if package is None:
                package = generate_package()
                package.owners.append(generate_person())
                ctx.udc.session.add(package)

            if release is None:
                package.releases.append(generate_release())

            ctx.udc.session.commit()

        elif action == "file_upload":
            content.name = os.path.basename(content.name)
            path = os.path.join(RootService.FILES_ROOT, pth)
            file_path = os.path.join(path, content.name)

            if package is None:
                package = generate_package()
                package.owners.append(generate_person())
                ctx.udc.session.add(package)

            if release is None:
                release = generate_release()
                package.releases.append(release)

            release.distributions.append(Distribution(
                    file_name=content.name,
                    file_path=pth,
                    dist_download_url=download_url,
                    dist_comment=comment,
                    dist_file_type=filetype,
                    dist_md5=md5_digest,
                    py_version=pyversion,
                    protocol_version=protcol_version,
                ))

            ctx.udc.session.flush()

            if os.path.isfile(file_path):
                raise ArgumentError((name, version  ))

            write_package_content(file_path)

            ctx.udc.session.commit()



# encoding: utf8
#
# (C) Copyright Arskom Ltd. <info@arskom.com.tr>
#               Uğurcan Ergün <ugurcanergn@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.
#

from collections import namedtuple

from sqlalchemy import sql

from sqlalchemy import create_engine
from sqlalchemy.orm import mapper
from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm import relation
from sqlalchemy.schema import UniqueConstraint

from spyne.model.complex import table
from spyne.model.complex import Array
from spyne.model.complex import TTableModel
from spyne.model.primitive import Date
from spyne.model.primitive import Integer32
from spyne.model.primitive import String
from spyne.model.primitive import Unicode

from spynepi.const import TABLE_PREFIX

DatabaseHandle = namedtuple("DatabaseHandle", ["db", "Session"])
TableModel = TTableModel()


class Person(TableModel):
    __tablename__ = "%s_person"  % TABLE_PREFIX
    __table_args__ = {"sqlite_autoincrement": True}

    id = Integer32(primary_key=True)
    person_name = String(256)
    person_email = String(256)


class Distribution(TableModel):
    __tablename__ = "%s_distribution"  % TABLE_PREFIX
    __table_args__ = {"sqlite_autoincrement": True}

    id = Integer32(primary_key=True)

    # TO-DO Add Content data
    file_name = String(256)
    file_path = String(256)
    dist_download_url = String(256)
    dist_comment = String(256)
    dist_file_type = String(256)
    dist_md5 = String(256)
    py_version = String(10)
    protocol_version = String(10)


class Release(TableModel):
    __tablename__ = "%s_release"  % TABLE_PREFIX
    __table_args__ = {"sqlite_autoincrement": True}

    id = Integer32(primary_key=True)
    release_cdate = Date
    rdf_about = String(256)
    release_version = String(10)
    meta_version = String(10)
    release_summary = String(256)
    release_platform = String(30)

    distributions = Array(Distribution).store_as(table(right="release_id"))


class Package(TableModel):
    __tablename__ = "%s_package"  % TABLE_PREFIX
    __table_args__ = (
        (UniqueConstraint("package_name"),),
        {"sqlite_autoincrement": True},
    )

    id = Integer32(primary_key=True)
    rdf_about = Unicode(256)
    package_name = String(40)
    package_cdate = Date
    package_description = Unicode
    package_license = Unicode(256)
    package_home_page = String(256)

    owners = Array(Person).store_as(table(right="owner_id"))
    releases = Array(Release).store_as(table(right="package_id"))


def patch_models():
    # this is here because the package_id column is not materialized until the
    # package table is created.
    Release.Attributes.sqla_table.append_constraint(
                            UniqueConstraint("package_id", "release_version"))

    package_m = Package.Attributes.sqla_mapper

    latest_releases = sql.select([sql.func.max(Release.id).label('release_id')],
                                     group_by=[Release.package_id]).alias()

    latest_release = Release.Attributes.sqla_table.select(
                Release.id == latest_releases.c.release_id).alias('latest_release')

    package_m.add_property('latest_release',
        relation(mapper(Release, latest_release, non_primary=True),
                uselist=False, viewonly=True)
    )

patch_models()


def init_database(connection_string):
    db = create_engine(connection_string)

    TableModel.Attributes.sqla_metadata.bind = db
    Session = sessionmaker(bind=db)

    # So that metadata gets updated with table names.
    import spynepi.entity.root
    import spynepi.entity.project

    TableModel.Attributes.sqla_metadata.create_all(checkfirst=True)
    return DatabaseHandle(db=db, Session=Session)



#!/usr/bin/env python

import os
import re

from setuptools import setup
from setuptools import find_packages

PROJECT_NAME = 'spynepi'

v = open(os.path.join(os.path.dirname(__file__), PROJECT_NAME, '__init__.py'), 'r')
VERSION = re.match(r".*__version__ = '(.*?)'", v.read(), re.S).group(1)

LONG_DESC = ""
try:
    os.stat('CHANGELOG.rst')
    LONG_DESC = open('CHANGELOG.rst', 'r').read()
except OSError:
    pass


setup(
    name=PROJECT_NAME,
    packages=find_packages(),
    version=VERSION,
    description="This is a caching PyPI implementation that uses Spyne",
    long_description=LONG_DESC,
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Operating System :: OS Independent',
        'Natural Language :: English',
        'Intended Audience :: Developers',
    ],
    keywords=('pypi', 'spyne', 'wsgi', 'rpc', 'http'),
    author='Ugurcan Ergun',
    author_email='ugurcanergn@gmail.com',
    url='http://github.com/arskom/spynepi',
    license='GPL',

    install_requires=[
        "spyne>=2.10","sqlalchemy<0.8", "werkzeug", "twisted", 'docutils'
    ],

    include_package_data=True,
    package_data={
        'spynepi.const.template': ['download.html'],
    },

    entry_points = {
        'console_scripts': [
            '%(p)s_daemon=%(p)s.main:main' % {'p': PROJECT_NAME},
        ]
    },
)




# The database connection string must be sqlalchemy compatible
DB_CONNECTION_STRING = "sqlite:///spynepi.db"

# The path(absolute recommended) where package files will be saved
FILES_PATH = 'files'

# The server address
HOST = "0.0.0.0"

# The server port
PORT = 7789

# The pypi repository name which defined in pypirc
REPO_NAME = "spynepi"

# The prefix for database tables
TABLE_PREFIX = "spynepi"
